/*******************************************************************************
 * Copyright (c) 2003 Rolf Wilms (rolf_wilms@hotmail.com).
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/cpl-v10.html
 * 
 * Contributors:
 * 
 *******************************************************************************/
package rwi.eclipse.team.ccvs.ssl;

import org.eclipse.jface.preference.FieldEditor;
import org.eclipse.jface.preference.FieldEditorPreferencePage;
import org.eclipse.jface.preference.FileFieldEditor;
import org.eclipse.jface.preference.RadioGroupFieldEditor;
import org.eclipse.jface.preference.StringFieldEditor;
import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPreferencePage;

/**
 * @author Rolf Wilms (rolf_wilms@hotmail.com)
 */
public class SSLPreferencePage
	extends FieldEditorPreferencePage
	implements IWorkbenchPreferencePage {

	private StringFieldEditor truststorePasswordEditor;

	private FileFieldEditor truststoreFileEditor;

	private StringFieldEditor keystorePasswordEditor;

	private FileFieldEditor keystoreFileEditor;

	private RadioGroupFieldEditor chooseClientKeysEditor;

	private RadioGroupFieldEditor chooseServerCertificatesEditor;

	private boolean useCustomKeystore;

	private boolean useCustomTruststore;

	public SSLPreferencePage() {
		super(GRID);
		setPreferenceStore(SSLPlugin.getDefault().getPreferenceStore());
		setDescription("Specify which certificates and keys to use for SSL connections to CVS");
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.preference.FieldEditorPreferencePage#createFieldEditors()
	 */
	protected void createFieldEditors() {
		chooseServerCertificatesEditor =
			new RadioGroupFieldEditor(
				SSLPlugin.KEY_SERVER_CERTIFICATES,
				"Server Certificates",
				3,
				new String[][] {
					{"Accept All", SSLPlugin.VAL_SERVER_CERTIFICATES_ACCEPT_ALL }, 
					{"Default Truststore", SSLPlugin.VAL_SERVER_CERTIFICATES_DEFAULT_TRUSTSTORE },
					{"Custom Truststore", SSLPlugin.VAL_SERVER_CERTIFICATES_CUSTOM_TRUSTSTORE }
		}, getFieldEditorParent(), false);
		addField(chooseServerCertificatesEditor);

		truststoreFileEditor =
			new FileFieldEditor(
				SSLPlugin.KEY_TRUSTSTORE_FILE,
				"Truststore File",
				true,
				getFieldEditorParent()) {
				public boolean isValid() {
					return (!useCustomTruststore) || super.isValid();
				}
			};
		truststoreFileEditor.setEmptyStringAllowed(false);
		addField(truststoreFileEditor);

		truststorePasswordEditor =
			new StringFieldEditor(
				SSLPlugin.KEY_TRUSTSTORE_PASSWORD,
				"Truststore Password",
				getFieldEditorParent());
		// turn into password editor
		truststorePasswordEditor.getTextControl(
			getFieldEditorParent()).setEchoChar(
			'*');
		addField(truststorePasswordEditor);


		chooseClientKeysEditor =
			new RadioGroupFieldEditor(
				SSLPlugin.KEY_CLIENT_KEYS,
				"Client Keys",
				2,
				new String[][] {
					{"Default Keystore", SSLPlugin.VAL_CLIENT_KEYS_DEFAULT_KEYSTORE },
					{"Custom Keystore", SSLPlugin.VAL_CLIENT_KEYS_CUSTOM_KEYSTORE }
		}, getFieldEditorParent(), false);
		addField(chooseClientKeysEditor);

		keystoreFileEditor =
			new FileFieldEditor(
				SSLPlugin.KEY_KEYSTORE_FILE,
				"Keystore File",
				true,
				getFieldEditorParent()) {
			public boolean isValid() {
				return (!useCustomKeystore) || super.isValid();
			}
		};
		keystoreFileEditor.setEmptyStringAllowed(false);
		addField(keystoreFileEditor);

		keystorePasswordEditor =
			new StringFieldEditor(
				SSLPlugin.KEY_KEYSTORE_PASSWORD,
				"Keystore Password",
				getFieldEditorParent());
		// turn into password editor
		keystorePasswordEditor.getTextControl(
			getFieldEditorParent()).setEchoChar(
			'*');
		addField(keystorePasswordEditor);
	}

	private void updateControls() {
		keystoreFileEditor.setEnabled(
			useCustomKeystore,
			getFieldEditorParent());
		keystorePasswordEditor.setEnabled(
			useCustomKeystore,
			getFieldEditorParent());
		truststoreFileEditor.setEnabled(
			useCustomTruststore,
			getFieldEditorParent());
		truststorePasswordEditor.setEnabled(
			useCustomTruststore,
			getFieldEditorParent());
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.IWorkbenchPreferencePage#init(org.eclipse.ui.IWorkbench)
	 */
	public void init(IWorkbench workbench) {
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.util.IPropertyChangeListener#propertyChange(org.eclipse.jface.util.PropertyChangeEvent)
	 */
	public void propertyChange(PropertyChangeEvent event) {

		if (event.getProperty().equals(FieldEditor.VALUE)) {
			if (event.getSource() == chooseClientKeysEditor) {
				useCustomKeystore =
				((String) event.getNewValue()).equals(
					SSLPlugin.VAL_CLIENT_KEYS_CUSTOM_KEYSTORE);
				updateControls();
			} else if (event.getSource() == chooseServerCertificatesEditor) {
				useCustomTruststore =
					((String) event.getNewValue()).equals(
						SSLPlugin.VAL_SERVER_CERTIFICATES_CUSTOM_TRUSTSTORE);
				updateControls();
			}
			// The validation aspect of the FieldEditor framework is sick. 
			// Always perform checkState when something changed. 
			checkState();
		}

		super.propertyChange(event);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.preference.FieldEditorPreferencePage#initialize()
	 */
	protected void initialize() {
		super.initialize();
		useCustomKeystore = SSLPlugin.getDefault().getUseCustomKeystore();
		useCustomTruststore = SSLPlugin.getDefault().getUseCustomTruststore();
		updateControls();
	}

}
